BitVector

Struct BitVector 

pub struct BitVector { /* private fields */ }
A cache-oblivious succinct bit vector.

Implementations

Source

impl BitVector

pub fn new(bits: &[u64], len: usize) -> Self

Create a new BitVector from a sequence of bits.

pub fn from_parts( storage: Vec<u64>, select1_index: Vec<u32>, select0_index: Vec<u32>, len: usize, ) -> Result<Self>

Reconstruct a BitVector from its internal parts.

This is primarily intended for serialization round-trips.

pub fn to_bytes(&self) -> Vec<u8>

Serialize this bitvector to a stable binary encoding (little-endian).

Format (versioned):

  • magic: 8 bytes (SBITBV01)
  • len: u64
  • storage_len: u64, then storage_len u64 words
  • select1_len: u64, then select1_len u32 words
  • select0_len: u64, then select0_len u32 words
pub fn from_bytes(bytes: &[u8]) -> Result<Self>

Deserialize a BitVector from to_bytes() output.

pub fn len(&self) -> usize

Return the total number of bits in the vector.

pub fn is_empty(&self) -> bool

Return true if the bit-vector has length 0.

pub fn heap_bytes(&self) -> usize

Approximate heap memory usage in bytes.

pub fn get(&self, i: usize) -> bool

Return true if the bit at index i is set.

pub fn rank1(&self, i: usize) -> usize

Return the number of set bits in the range [0, i).

pub fn rank0(&self, i: usize) -> usize

Return the number of unset bits in the range [0, i).

pub fn select1(&self, k: usize) -> Option<usize>

Return the position of the $k$-th set bit (0-indexed).

pub fn select0(&self, k: usize) -> Option<usize>

Return the position of the $k$-th unset bit (0-indexed).